--[[

	Add-On: CraftInfo
	
	This add-on is designed as an in-game front-end to the reagent database
	maintained by Jerigord (GDI), ReagentData. It allows looking up the reagents
	needed to craft any item in the ReagentData database (now including
	enchants!). It maintains a simple "shopping list" for reagents for a list
	of items and, optionally, decomposes any crafted reagents to their raw
	materials as well.
	
	NB: The value of 0.999 added for purposes of floor calculation (for
	minimum number of cycles to produce the needed items) is chosen in the
	event that someone wants to make 1 Crafted Solid Shot (or something
	similar), which would be 0.005 of a cycle.
	
	Dependencies: ReagentData (tested through version 2.4.5)
	
	Optional Dependencies: EnchantData

]]

local CraftInfo_items = {};
local CRAFTINFO_VERSION = "1.4.3";

-- saved variable to be used for loads
-- initialize to default values
CraftInfo_Config = {};
CraftInfo_Config.expandReagents = true;
CraftInfo_Config.showProfessions = true;
CraftInfo_Config.version = CRAFTINFO_VERSION;

------------------------------------------------------------------------------
--
-- CraftInfo_GetItem
-- Our main "get information" function. Given an item name:
-- 	Returns item
-- 	Returns nil if no matches
--
-- Note that Leather items are artifically terminated here, due to a chaining
-- issue that would reduce all leather to Ruined Leather Scraps.
--
local function CraftInfo_GetItem(item, exact)
	local lcitem, profession, citem;

	profession = next(ReagentData['professions'], nil);
	while profession ~= nil do
		if (ReagentData['crafted'][profession] == nil) then
			MK_Utils.LoadReagentDataProfession(profession);
		end
		if (ReagentData['crafted'][profession][item] ~= nil and
			string.find(item, " Leather$") == nil) then
			return profession, ReagentData['crafted'][profession][item];
		end
		profession = next(ReagentData['professions'], profession);
	end
	if (not exact) then
		-- if we get here, we didn't find anything; try lowercasing
		lcitem = string.lower(item);
		profession = next(ReagentData['professions'], nil);
		while profession ~= nil do
			citem = next(ReagentData['crafted'][profession], nil);
			while citem ~= nil do
				if (string.find(string.lower(citem), lcitem) ~= nil and
					string.find(citem, " Leather$") == nil) then
					return profession, ReagentData['crafted'][profession][citem];
				end
				citem = next(ReagentData['crafted'][profession], citem);
			end
			profession = next(ReagentData['professions'], profession);
		end
	end
	return nil, nil;
end; -- CraftInfo_GetItem

------------------------------------------------------------------------------
--
-- CraftInfo_GetEnchant
-- If EnchantData is loaded, looks up the string in the EnchantData
-- database and returns the item class the item was found in.
--
-- 	Returns table entry for item
-- 	Returns nil if no matches
--
local function CraftInfo_GetEnchant(item)
	if (EnchantData ~= nil) then
		for itemclass in EnchantData do
			if (type(EnchantData[itemclass]) == "table") then
				if (EnchantData[itemclass][item] ~= nil) then
					return 'enchanting', EnchantData[itemclass][item];
				end
			end
		end
	else
		if (ReagentData ~= nil and ReagentData['crafted'] ~= nil) then
			if (ReagentData['crafted']['enchanting'] == nil) then
				MK_Utils.LoadReagentDataProfession('enchanting');
			end
			if (ReagentData['crafted']['enchanting'] ~= nil) then
				return 'enchanting', ReagentData['crafted']['enchanting'][item];
			end
		end
	end
	return nil, nil;
end; -- CraftInfo_GetEnchant

local function CraftInfo_Lookup(item, quan)
	local itemName;
	local itemQuan;
	local itemProf;
	local itemRecord;
	local cycles;

	-- first check to see if this is an item or an item link
	if (item ~= nil and type(item) == "string") then
		if (string.find(item, "item:") ~= nil) then
			-- it's an item link! parse the name
			itemName = GetItemInfo(item);
			if (itemName == nil) then
				return nil, nil, nil, nil;
			end
		else
			itemName = item;
		end
		if (quan ~= nil and type(quan) == "number") then
			itemQuan = quan;
		else
			itemQuan = 1;
		end
	
		-- First see if it's an enchant
		itemProf, itemRecord = CraftInfo_GetEnchant(itemName);
		if (itemRecord == nil) then
			itemProf, itemRecord = CraftInfo_GetItem(itemName);
		end
		if (itemRecord ~= nil and itemRecord.reagents ~= nil and type(itemRecord.reagents) == "table") then
			itemTable = {};
			if (not itemRecord.result) then
				itemRecord.result = 1;
			end
			cycles = floor(itemQuan / itemRecord.result + 0.999);
			for r,n in pairs(itemRecord.reagents) do
				if (itemTable[r] ~= nil) then
					itemTable[r] = itemTable[r] + (cycles * n);
				else
					itemTable[r] = cycles * n;
				end
			end
			return itemName, itemProf, itemRecord.skill, itemTable;
		end
	end
	return nil, nil, nil, nil;
end -- CraftInfo_Lookup

local function CraftInfo_Print(frame)
	if (frame ~= nil) then
		local professions = {};
		local parts = {};
		local prof, part;
		local itemReagents;
		local reagents = {};
	
		frame:AddMessage("Crafter Summary:",
			0.0, 1.0, 1.0);
		for item, v in pairs(CraftInfo_items) do
			frame:AddMessage(string.format("%d %s", v.quantity, item));
			if (professions[v.profession] ~= nil) then
				if (v.skill > professions[v.profession]) then
					professions[v.profession] = v.skill;
				end
			else
				professions[v.profession] = v.skill;
			end
			_, _, _, itemReagents = CraftInfo_Lookup(item, v.quantity);
			if (itemReagents ~= nil) then
				for k, v in pairs(itemReagents) do
					if (reagents[k] ~= nil) then
						reagents[k] = reagents[k] + v;
					else
						reagents[k] = v;
					end
				end
			end
		end
		frame:AddMessage("Required Reagents:", 0.0, 1.0, 1.0);

		for r, v in pairs(reagents) do
			frame:AddMessage(string.format("%d %s", v, r));
			if (CraftInfo_Config.expandReagents) then
				prof, part = CraftInfo_GetItem(r, true);
				if (part ~= nil) then
					-- expandable reagent
					parts[r] = {};
					parts[r].profession = prof;
					parts[r].skill = part.skill;
					parts[r].quantity = v; 
					parts[r].reagents = part.reagents;
					parts[r].result = part.result;
				end
			end
		end
		if (CraftInfo_Config.showProfessions) then
			frame:AddMessage("Required Skills:", 0.1, 1.0, 1.0);
			for tradeskill, skill in pairs(professions) do
				frame:AddMessage(string.format("%s %d", ReagentData['professions'][tradeskill], skill));
			end
		end
		if (CraftInfo_Config.expandReagents and parts ~= {}) then
			local p;
			local dstuff;
			
			p = next(professions, nil);
			while p ~= nil do
				professions[p] = 0;
				p = next(professions, p);
			end
			dstuff = 0;
			repeat
				n = 0;
				for item, v in pairs(parts) do
					if (v.quantity > 0) then
						if (v.reagents ~= nil) then
							dstuff = 1;
							if (professions[v.profession] ~= nil) then
								if (v.skill > professions[v.profession]) then
									professions[v.profession] = v.skill;
								end
							else
								professions[v.profession] = v.skill;
							end
							p = next(v.reagents, nil);
							while p ~= nil do
								if (not v.result) then
									v.result = 1;
								end
								if (parts[p] ~= nil) then
									-- reagent is something we're already making
									parts[p].quantity = parts[p].quantity +
										floor(v.reagents[p] * v.quantity / v.result + 0.999);
								else
									prof, part = CraftInfo_GetItem(p, true);
									parts[p] = {};
									parts[p].quantity = floor(v.reagents[p] * v.quantity / v.result + 0.999);
									if (part ~= nil) then
										parts[p].profession = prof;
										parts[p].skill = part.skill;
										parts[p].reagents = part.reagents;
									end
								end
								n = n + 1;
								p = next(v.reagents, p);
							end
							v.quantity = 0;
						end
					end
				end
				p = next(parts, nil);
				while p ~= nil do
					if (parts[p].quantity == 0) then
						parts[p] = nil;
					end
					p = next(parts, p);
				end
			until n == 0
			
			if (dstuff ~= 0) then
				frame:AddMessage("Derived Reagents:", 0.0, 1.0, 1.0);
				for r, v in pairs(parts) do
					if (v.quantity > 0) then
						frame:AddMessage(string.format("%d %s", v.quantity, r));
					end
				end
				if (CraftInfo_Config.showProfessions) then
					frame:AddMessage("Derived Skills:", 0.1, 1.0, 1.0);
					for tradeskill, skill in pairs(professions) do
						frame:AddMessage(string.format("%s %d", ReagentData['professions'][tradeskill], skill));
					end
				end
			end
		end
	end
end -- CraftInfo_Print

------------------------------------------------------------------------------
-- Main routines
--

function CraftInfo_Command(msg)
	local n, m;
	local item, itemProf, itemSkill;
	local args

	if (msg ~= nil) then
		args = MK_Utils.ParseArgs(msg);
		n = 1;
		if (table.getn(args) == 0 or args[1] == "help") then
			-- print usage
			MK_Utils.Chat("Usage:");
			MK_Utils.Chat("/craft{info} {#} {'item name' or itemlink}");
			MK_Utils.Chat("     Adds the item (or multiple copies of the item) to the list of items being tracked");
			MK_Utils.Chat("/craft{info} show");
			MK_Utils.Chat("     Redisplays the craft list");
			MK_Utils.Chat("/craft{info} clear");
			MK_Utils.Chat("     Clears the tracking list");
			MK_Utils.Chat("/craft{info} {expand or noexpand}");
			MK_Utils.Chat("     (Don't) continue to expand crafted reagents");
			MK_Utils.Chat("/craft{info} {profs or noprofs}");
			MK_Utils.Chat("     (Don't) display required professions");
		else
			for i = 1, table.getn(args) do
				if (args[i] ~= nil) then
					if (args[i] == "clear") then
						CraftInfo_items = {};
					elseif (args[i] == "expand") then
						CraftInfo_Config.expandReagents = true;
						MK_Utils.Chat("CraftInfo now expanding derived reagents");
					elseif (args[i] == "noexpand") then
						CraftInfo_Config.expandReagents = false;
						MK_Utils.Chat("CraftInfo no longer expanding derived reagents");
					elseif (args[i] == "profs") then
						CraftInfo_Config.showProfessions = true;
						MK_Utils.Chat("CraftInfo now displaying required professions");
					elseif (args[i] == "noprofs") then
						CraftInfo_Config.showProfessions = false;
						MK_Utils.Chat("CraftInfo no longer displaying required professions");
					elseif (args[i] == "show") then
						CraftInfo_Print(DEFAULT_CHAT_FRAME);
					else
						m = tonumber(args[i]);
						if (m == nil) then
							item, itemProf, itemSkill, _ =
								CraftInfo_Lookup(args[i], n);
							if (item ~= nil) then
								if (CraftInfo_items[item] ~= nil) then
									CraftInfo_items[item].quantity =
										CraftInfo_items[item].quantity +
										n;
								else
									CraftInfo_items[item] = {};
									CraftInfo_items[item].quantity = n;
									CraftInfo_items[item].skill = itemSkill;
									CraftInfo_items[item].profession = itemProf;
								end
							else
								DEFAULT_CHAT_FRAME:AddMessage("CraftInfo: Could not find item '"..args[i].."'");
								-- add some kind of error message that item not found
							end
							n = 1;
						else
							n = m;
						end
					end
				end
			end -- for
			CraftInfo_Print(DEFAULT_CHAT_FRAME);
		end
	end
end -- CraftInfo_Command

function CraftInfo_OnEvent(event)
	if (event == "VARIABLES_LOADED") then
		SlashCmdList["CRAFTINFOCOMMAND"] = CraftInfo_Command;
		SLASH_CRAFTINFOCOMMAND1 = "/craftinfo";
		SLASH_CRAFTINFOCOMMAND2 = "/craft";

		if (DEFAULT_CHAT_FRAME) then
			DEFAULT_CHAT_FRAME:AddMessage("CraftInfo "..CRAFTINFO_VERSION..", ReagentData v."..ReagentData['version'].." loaded!");
		end
	end
end

function CraftInfo_OnLoad()
	this:RegisterEvent("VARIABLES_LOADED");
end -- CraftInfo_OnLoad
