--[[

	MK_Utils
	
	Various utility routines for MK Addons. This module creates the MK_Utils
	table which contains function pointers. This allows multiple addons to
	share the same routine set (quasi-shared libraries).

	Many thanks to Gazmik Fizzwidget (www.fizzwidget.com) for the idea (and a
	bunch of great addons!).

	API:

	MK_Utils.Chat(msgString)
		This routine displays the supplied message string in the default chat
		frame with the default color.

	MK_Utils.DChat(msgString)
		This routine displays the specified message string in the default chat
		frame only if debugging has been enabled (see Debug() below).

	MK_Utils.Debug(Boolean)
		This routine enables debug mode. Primarily, this enables print outs
		from everything using a DChatf routine to print.

	MK_Utils.LoadReagentDataProfession(profString)
		This routine calls the appropriate ReagentData_Load<profession> funciton
		to (re)load the database specified by name.

	argTable = MK_Utils.ParseArgs(msgString)
		This routine parses a text string and splits up the space-delimited
		entries into individual table entries. This is most useful in parsing
		arguments passed to a slash command, since all arguments are passed
		in a single string. The routine tries to be smart about parsing args
		enclosed in single or double quotes and makes all such enclosed phrases
		a single table entry. The routine also recognizes an item link and saves
		it in a form suitable for processing with GetItemInfo().
		This routine assumes that arguments are alphanumeric, possibly preceded
		by a hyphen.

	MK_Utils.ShowStack()
		Displays a message box with the current dynamic memory allocation to
		Lua variables in MB. Useful when debugging to check for memory leaks and
		in general to see if you should up your AddOn allocated memory.
]]

if (MK_Utils == nil) then
	MK_Utils = {};
end

if (MK_Utils.debug == nil) then
	MK_Utils.debug = false;
end

------------------------------------------------------------------------------
if (MK_Utils.Chat == nil) then
	MK_Utils.Chat = function(msg)
		DEFAULT_CHAT_FRAME:AddMessage(msg);
	end;
end -- MK_Utils.Chat

------------------------------------------------------------------------------
if (MK_Utils.DChat == nil) then
	MK_Utils.DChat = function(msg)
		if (MK_Utils.debug) then
			DEFAULT_CHAT_FRAME:AddMessage(msg);
		end;
	end;
end -- MK_Utils.DChat

------------------------------------------------------------------------------
if (MK_Utils.Debug == nil) then
	MK_Utils.Debug = function(doDebug)
		if (doDebug == nil or doDebug == true) then
			MK_Utils.debug = true;
		else
			MK_Utils.debug = false;
		end
	end;
end -- MK_Utils.Debug

------------------------------------------------------------------------------
if (MK_Utils.LoadReagentDataProfession == nil) then
	MK_Utils.LoadReagentDataProfession = function(profession)
		if (ReagentData ~= nil) then
			if (profession == 'alchemy') then
				ReagentData_LoadAlchemy();
			elseif (profession == 'blacksmithing') then
				ReagentData_LoadBlacksmithing();
			elseif (profession == 'cooking') then
				ReagentData_LoadCooking();
			elseif (profession == 'enchanting') then
				ReagentData_LoadEnchanting();
			elseif (profession == 'engineering') then
				ReagentData_LoadEngineering();
			elseif (profession == 'firstaid') then
				ReagentData_LoadFirstAid();
			elseif (profession == 'leatherworking') then
				ReagentData_LoadLeatherworking();
			elseif (profession == 'tailoring') then
				ReagentData_LoadTailoring();
			elseif (profession == 'jewelcrafting') then
				ReagentData_LoadJewelcrafting();
			else
				-- unknown profession supplied!
			end
		end
	end;
end -- MK_Utils.LoadReagentDataProfession

------------------------------------------------------------------------------
if (MK_Utils.ParseArgs == nil) then
	MK_Utils.ParseArgs = function(msg)
		local sS, sE, i, c;

		args = {};
		i = 1;
		repeat
			-- test for a phrase enclosed by double quotes
			sS, sE, c = string.find(msg, '^("[%w%s\']+")%s*', i);
			if (c == nil) then
				-- test for a phrase enclosed by single quotes
				sS, sE, c = string.find(msg, "^('[%w%s]+')%s*", i);
				if (c == nil) then
					-- test for a word, optionally including a leading hyphen
					sS, sE, c = string.find(msg, "^(%-?%w+)%s*", i);
					if (c == nil) then
						-- test for an item link (strip off header/trailer info)
						sS, sE, c = string.find(msg, "^|c%x+|H(item:%d+:%d+:%d+:%d+)|h%[.-%]|h|r%s*", i);
						if (c == nil) then
							break;
						end
					end
				else
					c = string.gsub(c, "'", "");
				end
			else
				c = string.gsub(c, '"', '');
			end
			table.insert(args, c);
			i = sE + 1;
		until false;
		return args;
	end;
end -- MK_Utils.ParseArgs

------------------------------------------------------------------------------
if (MK_Utils.ShowStack == nil) then
	MK_Utils.ShowStack = function()
		local stk = gcinfo();
		message("AddOns currently using: "..tostring(stk/1024.0).."M");
	end;
end -- MK_Utils.ShowStack
